"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const taskHandler_1 = require("./taskHandler");
const socket_1 = require("./socket");
/**
 * rpc framework master part
 * @author: lizhekang
 */
class Master extends taskHandler_1.default {
    /**
     * Master constructor
     * @param object obj with functions
     * @param config Master config
     */
    constructor(object, config) {
        super(object, config);
        this._cbMap = {};
    }
    init() {
        let cfg = this._config;
        this._server = new socket_1.Server(cfg.server);
        this._server.start();
        this._server.on('rpc', this._rpc.bind(this));
    }
    /**
     *
     * @param {string} name
     * @param {Array<any>} params
     * @returns {Promise}
     */
    call(name, params, cb, errCb) {
        if (typeof this._object[name] != 'function') {
            let res = new Result(-1, 'Function was not defined.');
            errCb(res.getResult());
        }
        //random
        //TODO: change to something new
        let map = this._server.getClientMap();
        let keys = Object.keys(map);
        if (keys.length > 0) {
            let index = Math.floor(Math.random() * keys.length);
            let client = map[keys[index]]; //choose socket client
            let rpcMsg = new socket_1.Message('rpc', 0, {
                name: name,
                params: params,
                callback: this._register(name, cb) //register random cb function
            });
            client.write(socket_1.DataHelper.getString(rpcMsg.getMessage()));
        }
        else {
            let res = new Result(-1, 'Not remote client is available.');
            errCb(res.getResult());
        }
    }
    _rpc(msg) {
        let data = msg.data;
        if (typeof this._cbMap[data.callback] == 'function') {
            this._cbMap[data.callback](data.result);
        }
    }
    /**
     * register the callback function
     * @param {string} name
     * @param {Function} callback
     * @private
     */
    _register(name, callback) {
        let cbName = name + '_' + new Date().getTime();
        this._cbMap[cbName] = callback;
        return cbName;
    }
    destroy() {
        this._server && this._server.destroy();
    }
}
class Result {
    constructor(result, msg, data = {}) {
        this._result = result;
        this._msg = msg;
        this._data = data;
    }
    getResult() {
        let res = {
            result: this._result,
            msg: this._msg,
            data: this._data
        };
        return JSON.parse(JSON.stringify(res));
    }
    set result(result) {
        this._result = result;
    }
    set msg(msg) {
        this._msg = msg;
    }
    set data(data) {
        this._data = data;
    }
}
exports.default = Master;
